extern crate xcb;
extern crate mpd;
extern crate simple_signal;
extern crate serde;
#[macro_use]
extern crate serde_derive;
extern crate toml;

extern crate freetype;
extern crate fontconfig_sys;

mod config;
mod ui;
mod widgets;
mod style;

use simple_signal::Signal;

use std::env;
use std::process;
use std::sync::mpsc;
use std::thread;
use std::time;

use ui::x11;
use widgets::Message;

fn main() {
    let config_path = env::args().nth(1).unwrap_or("./panel.toml".to_string());
    let cfg = config::load(&config_path);

    if let Some(conn) = x11::Connection::new() {
        let (tx, rx) = mpsc::channel();
        {
            let tx = tx.clone();
            simple_signal::set_handler(&[Signal::Int, Signal::Term], move |_signals| {
                tx.send(Message::Quit).expect("Failed to send quit");
            });
        }
        {
            let tx = tx.clone();
            let conn = conn.clone();
            thread::spawn(move || {
                loop {
                    match conn.wait_for_event() {
                        Some(event) => {
                            let message = Message::XcbEvent(event);
                            tx.send(message).expect("Failed to send xcb event");
                        },
                        None => { break; }
                    }
                }
            });
        }

        let mut panel = ui::panel::Panel::new(conn, &cfg);

        if let Some(&toml::Value::Array(ref configs)) = cfg.get("widget") {
            for config in configs {
                let type_ = config.get("type").and_then(|v| v.as_str()).unwrap();
                let align = config.get("align").and_then(|v| v.as_str()).unwrap_or("left");

                let params = widgets::WidgetParams {
                    id: "".to_string(),
                    context: panel.make_context(),
                    tx: tx.clone(),
                    config: config.clone()
                };

                if let Some(cons) = widgets::by_type(type_) {
                    let widget = cons(params.clone());
                    match align {
                        "right" => panel.add_right_widget(widget),
                        _ => panel.add_left_widget(widget)
                    }
                }
            }
        }

        panel.create();

        {
            let tx = tx.clone();
            thread::spawn(move || {
                loop {
                    tx.send(Message::Update).expect("Failed to send update");
                    thread::sleep(time::Duration::from_secs(1));
                }
            });
        }

        loop {
            let event = rx.recv();
            if panel.handle_event(event.unwrap()) {
                println!("Exiting");
                break;
            }
        }
    }

    process::exit(0);
}
