use std::fs::File;
use std::io::SeekFrom;
use std::io::prelude::*;

use style;
use ui::context::Context;
use widgets::{Message, Update, Widget, WidgetParams};

const ICON: &'static str = "";

pub struct Temperature {
    context: Context,
    file: File,
    temperature: Option<u32>
}

pub fn temperature(params: WidgetParams) -> Box<Widget> {
    let config: TemperatureConfig = params.config.try_into().unwrap();
    let path = format!("/sys/class/thermal/{}/temp", config.thermal_zone);
    let widget = Temperature {
        context: params.context,
        file: File::open(path).unwrap(),
        temperature: None
    };
    Box::new(widget)
}

impl Temperature {
    fn status(&self) -> String {
        match self.temperature {
            Some(i) => format!("{}°C", i/1000),
            None => "?°C".to_string(),
        }
    }
}

impl Widget for Temperature {
    fn render(&mut self, x: u16, width: u16) {
        style::render(&self.context, ICON, &self.status(), x, width);
    }

    fn width(&mut self) -> u16 {
        style::width(&self.context, ICON, &self.status())
    }

    fn handle_event(&mut self, event: &Message) -> Update {
        match event {
            &Message::Update => {
                let mut s = String::new();
                self.file.read_to_string(&mut s).ok().expect("Could not read temperature stats");
                let i : Option<u32> = s.trim().parse().ok();
                self.file.seek(SeekFrom::Start(0)).ok().expect("Could not reread temperature");

                let changed = i != self.temperature;
                self.temperature = i;

                if changed {
                    Update::Relayout
                }
                else {
                    Update::Nothing
                }
            },
            _ => Update::Nothing
        }
    }
}

#[derive(Deserialize)]
struct TemperatureConfig {
    thermal_zone: String
}
