extern crate time;

use style;
use ui::context::Context;
use widgets::{Message, Update, Widget, WidgetParams};

pub struct Time {
    context: Context,
    config: TimeConfig,
    icon: String,
    time: String
}

pub fn time(params: WidgetParams) -> Box<Widget> {
    let config: TimeConfig = params.config.try_into().unwrap();
    let icon = if config.is_utc {
        "UTC"
    }
    else {
        ""
    };
    let widget = Time {
        context: params.context,
        config: config,
        icon: icon.to_string(),
        time: "".to_string()
    };
    Box::new(widget)
}

impl Widget for Time {
    fn render(&mut self, x: u16, width: u16) {
        style::render(&self.context, &self.icon, &self.time, x, width);
    }

    fn width(&mut self) -> u16 {
        style::width(&self.context, &self.icon, &self.time)
    }

    fn handle_event(&mut self, event: &Message) -> Update {
        match event {
            &Message::Update => {
                let now =
                    if self.config.is_utc {
                        time::now_utc()
                    }
                    else {
                        time::now()
                    };

                let time = time::strftime(&self.config.format, &now);
                let new_time = time.unwrap();
                let changed = new_time != self.time;
                self.time = new_time;

                if changed {
                    Update::Relayout
                }
                else {
                    Update::Nothing
                }
            },
            _ => Update::Nothing
        }
    }
}

#[derive(Deserialize)]
struct TimeConfig {
    format: String,
    #[serde(default)]
    is_utc: bool
}
